/*
 * Copyright 2008 Dan Furie and Scott Linderman
 *
 * drawing.c contains methods for drawing to the lcd screen. It has methods 
 * for clearing the screen and drawing each screen of the menu. It also
 * has methods for selecting the current choice in the menu.
 */
 
//---------------------------------INCLUDES----------------------------------
//#include <stdio.h>

 //--------------------------------VARIABLES----------------------------------
 
 unsigned int color_bg = WHITE;
 unsigned int color_fg = BLACK;
 unsigned int color_sel = RED;
 unsigned int color_bar = BLUE;
 
 char y_padding = 10;
 char x_padding = 10;
 char item_padding = 2;
 char char_height = 7;
 char item_height = 11; //char_height + 2*item_padding
 char width = 112; //132 - 2 * x_padding;
 
 int i;  //loop counter
 
 //--------------------------------METHODS----------------------------------
  char flipy(char y)
  {
     //flip the y coord
     //input should be 0
     return 132 - y;
  }
 
  void clearScreen(void)
  {
  	/*
  	 * This method clears the screen by drawing the background color over it
  	 */
  	 LCD_draw_box(0,0,132,132,color_bg);
  }
  
  void drawScreen(int numChoices, char *choices[])
  {
  	/*
  	 * This method draws a menu screen with choices given in the choices[] array.
  	 * numChoices - number of menu items, length of choices
  	 * choices - array of string pointers
  	 */
  	 
  	 char y = y_padding;			//start menu items 10 px from top
  	 char x = x_padding + 2;		//start menu items 12 px from left
  	 
  	 for (i = 0; i < numChoices; i++)
  	 {
  	 	LCD_put_str(choices[i], color_fg, color_bg, x, y+item_padding);
  	 	y += item_height;
  	 }
  }
  
  void drawSave(void)
  {
  	/*
  	 * This method draws a save button at the bottom of a settings screen
  	 */
  	 char x = 41;
  	 char y = 132 - y_padding;
  	 char width = 30;
  	 
  	 //draw outline of button
  	 LCD_draw_box(x, y, width, 1, color_fg);
  	 LCD_draw_box(x, y + item_height - 1, width, 1, color_fg);
  	 LCD_draw_box(x, y, 1, item_height, color_fg);
  	 LCD_draw_box(x + width, y, 1, item_height, color_fg);
  	 
  	 //draw save string
  	 LCD_put_str("Save",color_fg, color_bg, x + 2, y + 2);
  }
  
  void selectSave(char erase)
  {
  	/*
  	 * This method either highlights or unhighlights the save button.
  	 */
    char x = 41;
    char y = 132 - x_padding;
    char width = 30;
  	 unsigned int color;
  	 if (erase == 1)
  	 	color = color_fg;
  	 else
  	 	color = color_sel;
  	 
  	 //draw outline of button
  	 LCD_draw_box(x, y, width, 1, color);
  	 LCD_draw_box(x, y + item_height - 1, width, 1, color);
  	 LCD_draw_box(x, y, 1, item_height, color);
  	 LCD_draw_box(x + width, y, 1, item_height, color);  	 
  }
  
  void drawMain(void)
  {
  	/*
  	 * This method draws the main screen menu choices
  	 */
     char *items[3] = {"Setup", "Start", "About"};
  	  drawScreen(3, items);
  }
  
  void drawSetup(void)
  {
  	/*
  	 * This method draws the main screen menu choices
  	 */
     char *items[2] = {"General Settings", "Shot Type"};
  	  drawScreen(2, items);
  }
  
  void drawStart(void)
  {
   /*
    * This method draws the choices for taking various shots
    */
     char *items[4] = {"Extended Exposure", "Audio", 
           "Accelerometer", "Photo"};
    drawScreen(4, items);
  }
  			
  void drawShotType(void)
  {
  	/*
  	 * This method draws the main screen menu choices
  	 */
     char *items[4] = {"Extended Exposure", "Audio Trigger", 
           "Accelerometer", "Light Trigger"}; 
  	  drawScreen(4, items);
  }
  
  void drawTakingShot()
  {
    /*
     * This method draws the generic screen for taking a shot. 
     */  
     
     //draw stop button
     char x = 41;
     char y = 132 - 2*y_padding;
     char width = 30;
      
     //draw outline of button
     LCD_draw_box(x, y, width, 1, color_sel);
     LCD_draw_box(x, y + item_height - 1, width, 1, color_sel);
     LCD_draw_box(x, y, 1, item_height, color_sel);
     LCD_draw_box(x + width, y, 1, item_height, color_sel);
      
     //draw stop string, its the only choice so its always selected
     LCD_put_str("Stop",color_fg, color_bg, x + 2, y + 2);
     
     //draw the outline of the progress bar
     x = 6;
     y = y_padding;
     width = 120;
     LCD_draw_box(x, y, width, 1, color_fg);
     LCD_draw_box(x, y + item_height - 1, width, 1, color_fg);
     LCD_draw_box(x, y, 1, item_height, color_fg);
     LCD_draw_box(x + width, y, 1, item_height, color_fg);
     
  }
  
  void drawDoneTakingShot()
  {
     /*
      * This method overwrites the stop button with an OK button
      * when the shot is complete. It also writes a Complete! message
      * over the progress indicator
      */
     char x = 41;
     char y = 132 - 2*y_padding;
     char width = 30;
     
     LCD_draw_box(x+2, y+1, width-2,item_height-2, color_bg);
     LCD_put_str("OK", color_fg, color_bg, x+14, y+2);
     
     //draw the outline of the progress bar
     x = 5;
     y = y_padding;
     width = 121;
     LCD_draw_box(x, y, width, 1, color_bg);
     LCD_draw_box(x, y + item_height - 1, width, 1, color_bg);
     LCD_draw_box(x, y, 1, item_height, color_bg);
     LCD_draw_box(x + width, y, 1, item_height, color_bg);
     
     //draw complete message
     LCD_put_str("Complete!",color_fg, color_bg, x+30, y+2);
  }
  
  void drawProgress(char prg, char erase)
  {
     /* 
      * This method draws a progress indicator that will scroll across the screen
      */
     char color;
     if (erase == 1) color = color_bg;
     else color = color_bar;
     char x = 6;
     char y = y_padding;
     char dx = 6;
     
     x += dx*prg;
     LCD_draw_box(x,y+1,dx,item_height-2,color);
  }
  
  void drawAbout(void)
  {
     /*
      * This method draws a simple about message
      */
     char x = x_padding;
     char y = y_padding;
     LCD_put_str("ECE 476 Spring 2008", color_fg, color_bg, x, y+item_padding);
     y += item_height;
     LCD_put_str("Dan Furie", color_fg, color_bg, x, y+item_padding);
     y += item_height;
     LCD_put_str("Scott Linderman", color_fg, color_bg, x, y+item_padding);
     y += item_height;
     y += item_height; 
     LCD_put_str("High Speed Photography", color_fg, color_bg, x, y+item_padding);
     y += item_height;  
     LCD_put_str("Controller with Sensors", color_fg, color_bg, x, y+item_padding);
  }
  
  void drawGenSettings(void)
  {
     /*
      * This draws the general settings
      * - use flash
      * - number of shots to take
      */
     
     char x = x_padding;
     char y = y_padding;
     
     char setting_offset = 70; //longer than strings
     
     //draw use flash
     LCD_put_str( "Flash Time:", color_fg, color_bg, x, y+item_padding);
     //draw checkbox
     LCD_draw_box(x+setting_offset, y, 28, 1, color_fg);
  	 LCD_draw_box(x+setting_offset, y + item_height - 1, 28, 1, color_fg);
  	 LCD_draw_box(x+setting_offset, y, 1, item_height, color_fg);
  	 LCD_draw_box(x+setting_offset + 28, y, 1, item_height, color_fg);
  	 //draw num shots
     y += item_height;
     LCD_put_str("Num Shots:", color_fg, color_bg, x, y+item_padding);
     //draw input box
     LCD_draw_box(x+setting_offset, y, 15, 1, color_fg);
  	 LCD_draw_box(x+setting_offset, y + item_height - 1, 15, 1, color_fg);
  	 LCD_draw_box(x+setting_offset, y, 1, item_height, color_fg);
  	 LCD_draw_box(x+setting_offset + 15, y, 1, item_height, color_fg);
  	 
  	 //draw save button
  	 drawSave();
  }
  
  void drawGenSettingsChoice(int choice, char erase)
  {
  	/*
  	 * This method draws a box around the current choice,
  	 * black if erasing old choice (erase = 1), red if selecting choice
  	 */
  	 char y =0;
  	 char x = 0; 
  	 char bwidth = 0;
  	 if (choice == 2)
  	 {
  	 	//defer drawing of save box to selectSave()
  	 	selectSave(erase);
  	 }
  	 else 
  	 {
	  	 if (choice == 0)
	  	 {
	  	 	//select "use flash"
	  	 	x = x_padding + 70;
	  	 	y = y_padding;
	  	 	bwidth = 28;
	  	 }
	  	 else if (choice == 1)
	  	 {
	  	 	//select "num shots"
	  	 	x = x_padding + 70;
	  	 	y = y_padding + item_height;
	  	 	bwidth = 15;
	  	 }
	  	 
	  	 //select correct color
	  	 unsigned int color;
	  	 if (erase == 1)
	  	 	color = color_fg;
	  	 else
	  	 	color = color_sel;
	  	 	
	  	 //draw box
	  	 LCD_draw_box(x, y, bwidth, 1, color);
	  	 LCD_draw_box(x, y + item_height - 1, bwidth, 1, color);
	  	 LCD_draw_box(x, y, 1, item_height, color);
	  	 LCD_draw_box(x + bwidth, y, 1, item_height, color);
  	 }
  }
  
  void changeFlashTime(unsigned int flashTime)
  {
  	/*
  	 * This method updates the flash time
  	 */
    //clear previous value
    LCD_draw_box(x_padding+70+1, y_padding+1, 26, item_height-2, color_bg);
    char flashTimeStr[4];
    sprintf(flashTimeStr,"%u",flashTime % 10000);
    LCD_put_str(flashTimeStr, color_fg, color_bg, x_padding + 72, y_padding + item_padding);
  }
  
  void changeNumShots(unsigned int numShots)
  {
  	/*
  	 * This method clears the previous num of shots and writes the new value
  	 */
  	 //clear previous value
  	 LCD_draw_box(x_padding+70+1, y_padding+item_height+1, 13, item_height-2, color_bg);
  	 char numShotsStr[2];
  	 sprintf(numShotsStr,"%u",numShots % 100);
  	 LCD_put_str(numShotsStr, color_fg, color_bg, x_padding + 72, y_padding + item_height + item_padding);
  }
  
  void drawExtendedExposure(void)
  {
     /*
      * This method draws the extended exposure settings 
      * - exposure time
      */
      
     char x = x_padding;
     char y = y_padding;
     
     char setting_offset = 55; //longer than strings
     
     //draw use flash
     LCD_put_str("Exp Time:", color_fg, color_bg, x, y+item_padding);
     //draw exposure time (4 digit spaces)
     LCD_draw_box(x+setting_offset, y, 30, 1, color_fg);
  	 LCD_draw_box(x+setting_offset, y + item_height - 1, 30, 1, color_fg);
  	 LCD_draw_box(x+setting_offset, y, 1, item_height, color_fg);
  	 LCD_draw_box(x+setting_offset + 30, y, 1, item_height, color_fg);
  	 
  	 //draw save button
  	 drawSave();
  }
  
  void drawExposureTimeChoice(int choice, char erase)
  {
  	/*
  	 * This method draws a box around the current choice,
  	 * black if erasing old choice (erase = 1), red if selecting choice
  	 */
  	 char y = 0; 
  	 char x = 0; 
  	 char bwidth = 0;
  	 
  	 if (choice == 0)
  	 {
  	 	//select "exposure time"
  	 	x = x_padding + 55;
  	 	y = y_padding;
  	 	bwidth = 30;
  	 
  	 
  	 	//select correct color
	  	unsigned int color;
	  	if (erase == 1)
	  		color = color_fg;
	  	else
	  		color = color_sel;
	  		
	  	//draw box
	  	LCD_draw_box(x, y, bwidth, 1, color);
	  	LCD_draw_box(x, y + item_height - 1, bwidth, 1, color);
	  	LCD_draw_box(x, y, 1, item_height, color);
	  	LCD_draw_box(x + bwidth, y, 1, item_height, color);
  	 }
  	 else
  	 {
  	 	selectSave(erase);
  	 }
  }
  
  void changeExposureTime(unsigned int seconds)
  {
  	/*
  	 * This method clears the previous num of shots and writes the new value
  	 */
  	 //clear previous value
  	 LCD_draw_box(x_padding+56, y_padding+item_height+1, 28, item_height-2, color_bg);
  	 char secondsStr[4];
  	 sprintf(secondsStr,"%u",seconds);
  	 LCD_put_str(secondsStr, color_fg, color_bg, x_padding + 57, y_padding + item_padding);
  }
  

  void drawAudioTrigger(void)
  {
     /*
      * This method draws the audio trigger settings
      * - threshold 
      */
      
     //draw an analog meter on the left side
     drawAnalogMeter();
  	 
  	 drawSave();
  }
  
  void drawAccelerometer(void)
  {
     /* 
      * This method draws the accelerometer trigger settings
      * - threshold
      */
     //draw an analog meter on the left side
     drawAnalogMeter();
  	 
  	 drawSave();
  }
  
  void drawPhotoTrigger(void)
  {
     /*
      * This method draws the photo trigger settings
      * - threshold
      */
     //draw an analog meter on the left side
     drawAnalogMeter();
  	 
  	 drawSave();
  }
  
  void drawAnalogMeter(void)
  {
  	/*
  	 * This method draws a standard analog meter on the left side of the screen.
  	 */
  	//draw an analog meter on the left side with an 
     //arrow to indicate current threshold
    LCD_draw_box(x_padding, 1, 20, 1, color_fg);
  	 LCD_draw_box(x_padding, 1 + 130, 20, 1, color_fg);
  	 LCD_draw_box(x_padding, 1, 1, 130, color_fg);
  	 LCD_draw_box(x_padding + 20, 1, 1, 130, color_fg);
  }
  
  void updateAnalogMeter(unsigned char val) {
    val = val>>1;
    if (val==3) {
     val--;
    } else if (val>3) {
     val = val - 2;
    }
    LCD_draw_box(x_padding+1,1,19,128,color_bg);
    LCD_draw_box(x_padding+1,1,19,val,color_bar);
  }
  
  void drawThresholdMarker(unsigned char thresh, char erase)
  {
  	/*
  	 * This method draws the threshold indicator '<' at the appropriate
  	 * levl by converting val [0:255] to a position on the meter
  	 * 
  	 * if erase == 0: draw in selected color
  	 * if erase == 1: draw in fg color
  	 * otherwise draw in bg color
  	 */
  	 
  	 unsigned int color;
  	 if (erase == 0)
  	 	color = color_sel;
  	 else if (erase == 1)
  	 	color = color_fg;
  	 else 
  	 	color = color_bg;
  	 
  	 unsigned char x = x_padding + 22;	//draw to the right of the meter
  	 unsigned char h = (thresh >> 1); // (thresh / 255) * 100
  	 unsigned char y = 130 - h - 3;	// 1 + 128 - h is top of meter bar, 
  	 									//-3 so center of arrow points to thresh 
  	 
  	 LCD_put_str("<",color,color_bg,x,y);
  }
  
  void drawAnalogMeterChoice(int choice, unsigned char thresh, char erase)
  {
  	/*
  	 * This method switches the choice between the save button
  	 * and the analog meter
  	 */
  	 if (choice == 0)
  	 {
  	 	//only change the threshold marker in this method
  	 	//defer save button to selectSave()
	  	drawThresholdMarker(thresh,erase);
  	 }
  	 else
  	 {
  	 	selectSave(erase);
  	 }
  }
  
  void clearChoice(int oldChoice)
  {
  	/*
  	 * Erase the old choice's rectangle by drawing a rectangle around it in bg color
  	 */
  	 char y = y_padding + oldChoice * item_height;
  	 //erase top
  	 LCD_draw_box(x_padding, y, width, 1, color_bg);
  	 //erase bottom
  	 LCD_draw_box(x_padding, y + item_height - 1, width, 1, color_bg);
  	 //erase left
  	 LCD_draw_box(x_padding, y, 1, item_height, color_bg);
  	 //erase right
  	 LCD_draw_box(x_padding + width, y, 1, item_height, color_bg);
  }
  
  void drawChoice(int newChoice)
  {
  	/*
  	 * Select a choice by drawing a rectangle around it in fg color
  	 */
  	 char y = y_padding + newChoice * item_height;
  	 //erase top
  	 LCD_draw_box(x_padding, y, width, 1, color_fg);
  	 //erase bottom
  	 LCD_draw_box(x_padding, y + item_height - 1, width, 1, color_fg);
  	 //erase left
  	 LCD_draw_box(x_padding, y, 1, item_height, color_fg);
  	 //erase right
  	 LCD_draw_box(x_padding + width, y, 1, item_height, color_fg);
  }

