void waitms(int ms) {
  // Imprecise timer used during the initilization of the LCD.
  int i;
  for (i=0;i<ms;i++) _delay_ms(1);
}

void lcdInit (void) {
  SPIDDR=(1<<SDA)|(1<<CLK)|(1<<CS)|(1<<RESET); //Port-Direction Setup
  
  int   i = 0;

  CS0
  SDA0
  CLK1

  RESET1
  RESET0
  RESET1

  CLK1
  SDA1
  CLK1

  waitms(10);

  //Software Reset
  sendCMD(DISCTL);
  
  //added
  sendData(0);
  sendData(32);
  sendData(0);
  sendData(0x00);
  
  sendCMD(COMSCN);  // comscn
  sendData(1);

  sendCMD(OSCON);  // oscon

  sendCMD(SLPOUT);  // sleep out

  sendCMD(VOLCTR);  // electronic volume, this is kinda contrast/brightness
  sendData(30);//ff);       //  this might be different for individual LCDs
  sendData(3);//01);     //

  sendCMD(PWRCTR);  // power ctrl
  sendData(0x0f);      //everything on, no external reference resistors
  waitms(100);

  sendCMD(DISINV);  // DISINV


  sendCMD(DATCTL);  // datctl
  sendData(0x03);
  sendData(0);
  sendData(0x04);
  //  sendData(0x00);
  sendCMD(NOP);  // nop
  
  
  sendCMD(PASET);   // page start/end ram
  sendData(2);            // for some reason starts at 2
  sendData(131);          
  
  sendCMD(CASET);   // column start/end ram
  sendData(0);          
  sendData(131);
 
  sendCMD(RAMWR);    // write some stuff (background)
  for (i = 0; i < 18000; i++){
    sendData(0x0f);  // This makes the default background on startup WHITE
	sendData(0xff);
  }
  sendCMD(NOP);
  

  waitms(100);
  sendCMD(DISON);   // display on

  waitms(200);

  
  for (i = 0; i < 1; i++){   // this loop adjusts the contrast, change the number of iterations to get
	sendCMD(VOLUP);                // desired contrast.  This might be different for individual LCDs
	waitms(2);
  }
}



void shiftBits(byte b) {

  CLK0
  if ((b&128)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&64)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&32)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&16)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&8)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&4)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&2)!=0) SDA1 else SDA0
  CLK1

  CLK0
  if ((b&1)!=0) SDA1 else SDA0
  CLK1
}

void LCD_put_bitmap(unsigned int *bitmap, byte x, byte y, byte width, byte height) {
  // blits a bitmap to the screen from the provided array.
  byte color1,color2;

  setDrawingArea(x,y,width,height);
  sendCMD(RAMWR);    // write some shit
  
  byte i;
  unsigned int size = width * height;
  for (i=0;i<size;i++) {
	color1 = bitmap[size-i-1]>>8;
	color2 = bitmap[size-i-1] & 0xff;
	sendData(color1);
	sendData(color2);
  }
  sendCMD(NOP);
  
}

void LCD_put_bitmapf(unsigned int *bitmap, byte x, byte y, byte width, byte height) {
  // Same as the function above but designed to work with bitmaps stored in FLASH.
  byte color1,color2;

  setDrawingArea(x,y,width,height);
  sendCMD(RAMWR);    

  unsigned int i;
  unsigned int size = width * height;
  unsigned int tmp;

  for (i=0;i<size;i++) {
	tmp = pgm_read_word(&(bitmap[i]));
	color1 = tmp>>8;
	color2 = tmp & 0xff;
	sendData(color1);
	sendData(color2);
  }
  sendCMD(NOP);
  
}

void setDrawingArea(byte x, byte y, byte width, byte height) {
  x+=2;
  sendCMD(CASET);   // page start/end ram
  sendData(y);            
  sendData(y+height-1);          
  sendCMD(PASET);   // column start/end ram
  sendData(x);            // for some reason starts at 2
  sendData(x+width-1); 
}

void LCD_draw_box(byte x, byte y, byte width, byte height, unsigned int color) {
  byte color1,color2;
  setDrawingArea(x,y,width,height);
  sendCMD(RAMWR);
  
  color1 = color>>8;
  color2 = color & 0xff;

  unsigned int i;
  unsigned int size = width * height;

  for (i=0;i<size;i++) {
	sendData(color1);
	sendData(color2);
  }
  sendCMD(NOP);

}

void LCD_put_str(char *str, unsigned int color_fg, unsigned int color_bg, byte x, byte y) {
  byte tmpX,i;
  i = 0;
  tmpX = x;

  while (str[i] != '\0') {
	LCD_put_char(str[i], color_fg, color_bg, tmpX, y);
	i++;
	tmpX+=6;
  }
}

void LCD_put_char(byte c, unsigned int color_fg, unsigned int color_bg, byte x, byte y) {
  byte color_fg1,color_fg2,color_bg1,color_bg2;
  color_fg1 = color_fg>>8;
  color_fg2 = color_fg & 0xff;
  color_bg1 = color_bg>>8;
  color_bg2 = color_bg & 0xff;

  unsigned int startIndex;
  startIndex = (c - 32)*5;

  setDrawingArea(x,y,5,7);
  sendCMD(RAMWR);

  byte i,offset,tmpByte;
  for (i=0;i<5;i++) {
	tmpByte = pgm_read_byte(&(Font5x7[startIndex+i]));
	for (offset=0;offset<7;offset++) {
	  if (tmpByte & 0x40) {
		// put FG color
		sendData(color_fg1);
		sendData(color_fg2);
	  } else {
		// put BG color
		sendData(color_bg1);
		sendData(color_bg2);
	  }
	  tmpByte = tmpByte<<1;
	}
  }
}

void LCD_put_pixel(byte r, byte g, byte b, byte x, byte y){
  byte color1,color2;
  color1 = (r & 0x0f);
  color2 = (g<<4) | (b & 0x0f);
  
  setDrawingArea(x,y,1,1);
  sendCMD(RAMWR);    
  sendData(color1);
  sendData(color2);
  sendCMD(NOP);
}

//send data
void sendData(byte data) {
  
  CLK0
  SDA1                                                 //1 for param
  CLK1
	
  shiftBits(data);
}

//send cmd
void sendCMD(byte data) {

  CLK0
  SDA0                                                 //1 for cmd
  CLK1

  shiftBits(data);
}
